/*** 
 * @author Dave Mulkey
 * @version Dec 13 2015   Germany
 ***/

//Easy GUI Apps, Components, IBIO
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import javax.swing.plaf.FontUIResource;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

public class GUI extends JFrame implements ActionListener, 
ItemListener, MouseListener, KeyListener, ChangeListener
{
    MenuBar menubar = addMenuBar();

    public GUI()
    {
        setLayout(new FlowLayout());
        setSize(600,500);
        setup();
    }

    void setup()
    {
        addWindowListener                    // enable [x] for closing app
        ( new WindowAdapter()
            { public void windowClosing(WindowEvent evt)
                { evt.getWindow().dispose(); }
            }
        );
        setVisible(true);
        //UIManager.put("Label.font", new FontUIResource(new Font("Dialog", 0, 18)));
        //UIManager.put("Button.font", new FontUIResource(new Font("Dialog", Font.BOLD, 18)));       
        toFront();
        requestFocus();
    }

    public GUI(int w, int h)
    {
        setLayout(null);
        setSize(w,h);
        setup();
    }   

    public Dimension getPreferredSize()
    {
        return getSize();
    }

    public MenuBar makeMenus(String[] menus)
    {
        for (int x=0;x<menus.length;x++)
        {  if (menus[x].length()>0)
            { int d = menus[x].indexOf("|");
                if (d<0) {d=menus[x].length();  }
                Menu newmenu = addMenu(menus[x].substring(0,d),menubar);

                while (d>=0 && d<menus[x].length())
                {
                    int e = menus[x].indexOf("|",d+1);
                    if (e<0) {e=menus[x].length();  }
                    MenuItem newmi = addMenuItem(menus[x].substring(d+1,e),newmenu,this);
                    newmi.setActionCommand(newmenu.getActionCommand()+"|"+newmi.getActionCommand());

                    d = e;
                }
            }
        }
        return menubar;
    }

    //============================
    //--- Control Creators -------
    //============================

    public Button addButton(String text, int left, int top, int width, int height,
    ActionListener listener)
    {
        Button newComp = new Button(text);
        add(newComp);
        newComp.setBounds(left,top,width,height);
        newComp.setFont(bigFont);
        if (listener!=null)newComp.addActionListener(listener);
        return newComp;
    }

    public Button addButton(String text)
    {
        Button newComp = new Button(text);
        add(newComp);
        newComp.setFont(bigFont);
        newComp.addActionListener(this);
        return newComp;
    }

    public Checkbox addCheckbox(String text, int left, int top, int width, int height,
    ItemListener listener)
    {
        Checkbox newComp = new Checkbox(text);
        add(newComp);
        newComp.setBounds(left,top,width,height);
        newComp.setFont(bigFont);
        if (listener!=null)newComp.addItemListener(listener);
        return newComp;
    }

    public TextField addTextField(String text, int left, int top, int width, int height,
    ActionListener listener)
    {
        TextField newComp = new TextField(text);
        add(newComp);
        newComp.setBounds(left,top,width,height);
        newComp.setFont(bigFont);
        if (listener!=null)newComp.addActionListener(listener);
        return newComp;
    }

    public TextField addTextField(String text)
    {
        TextField newComp = new TextField(text);
        add(newComp);
        newComp.setFont(bigFont);
        newComp.addActionListener(this);
        return newComp;
    }

    public TextArea addTextArea(String text, int left, int top, int width, int height,
    KeyListener listener)
    {
        TextArea newComp = new TextArea(text);
        add(newComp);
        newComp.setBounds(left,top,width,height);
        newComp.setFont(bigFont);
        if (listener!=null)newComp.addKeyListener(listener);
        return newComp;
    }

    public TextArea addTextArea(String text)
    {
        TextArea newComp = new TextArea(text);
        add(newComp);
        newComp.setPreferredSize(new Dimension(350,100));
        newComp.setFont(bigFont);
        newComp.addKeyListener(this);
        return newComp;
    }

    public Label addLabel(String text, int left, int top, int width, int height,
    MouseListener listener)
    {
        Label newComp = new Label(text);
        add(newComp);
        newComp.setBounds(left,top,width,height);
        newComp.setFont(bigFont);
        if (listener!=null)newComp.addMouseListener(listener);
        return newComp;
    }

    public Label addLabel(String text)
    {
        Label newComp = new Label(text);
        add(newComp);
        newComp.setFont(bigFont);
        newComp.addMouseListener(this);
        return newComp;
    }

    public JLabel addJLabelURL(String urlName, int left, int top, int width, int height,
    MouseListener listener)
    {
        JLabel newComp = new JLabel();
        loadURLImage(newComp, urlName);
        add(newComp);
        newComp.setBounds(left,top,width,height);
        newComp.setFont(bigFont);
        if (listener!=null)newComp.addMouseListener(listener);
        return newComp;
    }

    public JLabel addJLabel(String fileName, int left, int top, int width, int height,
    MouseListener listener)
    {
        JLabel newComp = new JLabel();
        loadImage(newComp, fileName);
        add(newComp);
        newComp.setBounds(left,top,width,height);
        newComp.setFont(bigFont);
        if (listener!=null)newComp.addMouseListener(listener);
        return newComp;
    }      

    public List addList(String words,int left, int top, int width, int height,
    ItemListener listener)
    {
        List newComp = new List();
        while (words.length()>0)
        { int s = words.indexOf("|");
            if (s<0)
            { newComp.add(words);
                words = "";
            }
            else
            { newComp.add(words.substring(0,s));
                words = words.substring(s+1);
            }
        }
        add(newComp);
        newComp.setBounds(left,top,width,height);
        newComp.setFont(bigFont);
        if (listener!=null)newComp.addItemListener(listener);
        return newComp;
    }

    public List addList(String words)
    {
        List newComp = new List();
        while (words.length()>0)
        { int s = words.indexOf("|");
            if (s<0)
            { newComp.add(words);
                words = "";
            }
            else
            { newComp.add(words.substring(0,s));
                words = words.substring(s+1);
            }
        }
        add(newComp);
        newComp.setFont(bigFont);
        newComp.addItemListener(this);
        return newComp;
    }

    public Choice addChoice(String words,int left, int top, int width, int height,
    ItemListener listener)
    {
        Choice newComp = new Choice();
        while (words.length()>0)
        { int s = words.indexOf("|");
            if (s<0)
            { newComp.add(words);
                words = "";
            }
            else
            { newComp.add(words.substring(0,s));
                words = words.substring(s+1);
            }
        }
        add(newComp);
        newComp.setBounds(left,top,width,height);
        newComp.setFont(bigFont);
        if (listener!=null)newComp.addItemListener(listener);
        return newComp;
    }

    public Choice addChoice(String words)
    {
        Choice newComp = new Choice();
        while (words.length()>0)
        { int s = words.indexOf("|");
            if (s<0)
            { newComp.add(words);
                words = "";
            }
            else
            { newComp.add(words.substring(0,s));
                words = words.substring(s+1);
            }
        }
        add(newComp);
        newComp.setFont(bigFont);
        newComp.addItemListener(this);
        return newComp;
    }

    public MenuItem addMenuItem(String mText,Menu mu,ActionListener listener)
    {  MenuItem c = new MenuItem(mText);
        mu.add(c);
        if (listener!=null)c.addActionListener(listener);
        return c;
    }

    public Menu addMenu(String mText,Menu mu)
    {
        Menu c = new Menu(mText);
        mu.add(c);
        return c;
    }

    public Menu addMenu(String mText,MenuBar mb)
    {
        Menu c = new Menu(mText);
        mb.add(c);
        return c;
    }

    public Menu addMenu(String mText)
    {
        int d = mText.indexOf("|");
        if (d<0)
        {d=mText.length();}
        String id = mText.substring(0,d);
        Menu c = new Menu(id);
        c.setFont(bigFont);
        menubar.add(c);
        while (d<mText.length())
        { int e = mText.indexOf("|",d+1);
            if (e<0)
            {e = mText.length();}
            String cmd = mText.substring(d+1,e);
            MenuItem mi = addMenuItem(cmd,c,this);
            mi.setActionCommand(id+"|"+cmd);
            c.add(mi);
            d = e;
        }
        return c;
    }

    public MenuBar addMenuBar()
    {  MenuBar c = new MenuBar();
        setMenuBar(c);
        return c;
    }

    //==================================
    //=== Simplified Command Library ===
    //==================================

    public Process runProgram(String progName)
    {
        Process handle = null; 

        if(System.getProperty("os.name").substring(0,1).equalsIgnoreCase("W"))
        {   // Windows
            try
            { handle = Runtime.getRuntime().exec(progName);
            }
            catch(IOException ex)
            { }
        }
        else  
        {
            // Mac
            try{
                if(progName.indexOf("|")>0)
                {  String com1 = progName.substring(0,progName.indexOf("|"));
                    String com2 = progName.substring(progName.indexOf("|")+1);
                    String[] cmdline = { "open", "-a", com1 , com2};
                    handle = Runtime.getRuntime().exec(cmdline);
                }
                else
                {
                    String[] cmdline = { "open", "-a", progName};
                    handle = Runtime.getRuntime().exec(cmdline);
                }
            }
            catch(Exception ex){ }
        }
        return handle;
    }

    public String chooseFile()
    { FileDialog fd = new FileDialog(this);
        fd.setVisible(true);
        return fd.getDirectory() + fd.getFile();
    }

    public void loadImage(JLabel jl, String filename)
    { if(filename.length()>0)
        {      
            ImageIcon im = new ImageIcon(filename);
            jl.setIcon(im);
        }
    }

    public void loadURLImage(JLabel jl, String urlName)
    {  
        try
        {
            java.net.URL url = new java.net.URL(urlName);
            java.awt.image.BufferedImage bim = javax.imageio.ImageIO.read(url);
            ImageIcon im = new ImageIcon(bim);

            //        int c = 0;
            //        do
            //        { c++;
            //        } while (c<100000 && im.getImageLoadStatus()!=MediaTracker.COMPLETE);
            jl.setIcon(im);
        } catch(Exception ex){ }
    }  

    public boolean loadList(List list,String filename)
    {
        try
        { list.removeAll();
            BufferedReader infile = new BufferedReader(new FileReader(filename));
            while(infile.ready())
            { list.add(infile.readLine());
            }
            infile.close();
            return true;
        }
        catch(IOException e)
        { output(e.toString());
            return false;
        }
    }

    public boolean saveList(String filename,List list)
    {
        try
        { PrintWriter outfo = new PrintWriter(new FileWriter(filename));
            for (int c=0;c<list.getItemCount();c++)
            { outfo.println(list.getItem(c));
            }
            outfo.close();
            return true;
        }
        catch(IOException e)
        { output(e.toString());
            return false;
        }
    }

    public boolean loadArray(String[] array,String filename)
    {
        try
        {
            BufferedReader countFile = new BufferedReader(new FileReader(filename));
            int s = 0;
            while(countFile.ready())
            { array[s] = countFile.readLine();
                s++;
            }
            array[s] = null;
            countFile.close();
            return true;
        }
        catch(Exception e)
        { output(e.toString());
            return false;
        }
    }

    public boolean saveArray(String filename,String[] array)
    {
        try
        { PrintWriter outfo = new PrintWriter(new FileWriter(filename));
            int c = 0;
            while (array[c]!=null)
            { outfo.println(array[c]);
                c++;
            }
            outfo.close();
            return true;
        }
        catch(IOException e)
        { output(e.toString());
            return false;
        }
    }

    public void sortArray(String[] arr)
    {
        int swaps = 0 ;
        do
        { swaps = 0;
            int c = 0;
            while (arr[c+1]!=null)
            { if (arr[c].compareTo(arr[c+1])>0)
                { String temp = arr[c];
                    arr[c] = arr[c+1];
                    arr[c+1] = temp;
                    swaps++;
                }
                c++;
            }
        } while (swaps != 0);
    }

    public String getType(Object ob)
    {
        String c = ob.getClass().toString();
        int p = c.lastIndexOf(".");
        if (p<0) {p=-1;}
        return c.substring(p+1);
    }

    public void showFile(String fileName)
    {
        String os = System.getProperty("os.name").substring(0,3); 
        if(os.equals("Win"))
        {
            try{
                runProgram("explorer " + fileName);
            }
            catch(Exception ex)
            {
                System.out.println(ex.toString());  
            }
        }    
        if(os.equals("Mac") || os.equals("Lin"))
        {
            try{
                Runtime.getRuntime().exec(
                    new String[]{"/usr/bin/open",
                        fileName}
                );
            }
            catch(Exception ex)
            {  System.out.println(ex.toString()); }
        }    

    }

    //=== Stub versions of Listeners to satisfy interfaces ===
    public void actionPerformed(ActionEvent evt)
    { actions(evt.getSource(),evt.getActionCommand());
    }

    public void itemStateChanged(ItemEvent evt)
    { Object source = evt.getSource();
        String t = getType(source);
        String command = "";
        if (t.equals("List")) { command = ((List)source).getSelectedItem(); }
        else if (t.equals("Choice")){ command = ((Choice)source).getSelectedItem(); }
        else {command = evt.getItem().toString();}
        actions(source,command);
    }

    public void stateChanged(javax.swing.event.ChangeEvent evt)
    {
        Object source = evt.getSource();
        String t = getType(source);
        String command = "";
        command = evt.toString();
        actions(source,command);
    }

    public void actions(Object source,String command)
    {
        output(source.toString()+":"+command);
    }

    public void mouseClicked(MouseEvent e)
    { Object source = e.getSource();
        actions(source,"");
    }

    public void mouseEntered(MouseEvent e){}

    public void mouseExited(MouseEvent e){}

    public void mousePressed(MouseEvent e){}

    public void mouseReleased(MouseEvent e){}

    public void keyTyped(KeyEvent e)
    { Object source = e.getSource();
        actions(source,e.getKeyChar()+"");
    }

    public void keyPressed(KeyEvent e){}

    public void keyReleased(KeyEvent e){}

    //===========================================================
    //=== IBIO simplified input/output commands - GUI version ===
    //===========================================================

    public void outputString(String message)
    {  output(message);  }

    public void output(char info)
    {  output(info + ""); }

    public void output(byte info)
    {  output(info + ""); }

    public void output(int info)
    {  output(info + ""); }

    public void output(long info)
    {  output(info + ""); }

    public void output(double info)
    {  output(info + ""); }

    public void output(boolean info)
    {  output(info + ""); }

    //----- Numerical input methods return 0 on error ------------

    public String inputString(String prompt)
    { return input(prompt);  }

    public String input()
    { return input("");     }

    public char inputChar(String prompt)
    { char result=(char)0;
        try{result=input(prompt).charAt(0);}
        catch (Exception e){result = (char)0;}
        return result;
    }

    public byte inputByte(String prompt)
    { byte result=0;
        try{result=Byte.valueOf(input(prompt).trim()).byteValue();}
        catch (Exception e){result = 0;}
        return result;
    }

    public int inputInt(String prompt)
    {  int result=0;
        try{result=Integer.valueOf(
                input(prompt).trim()).intValue();}
        catch (Exception e){result = 0;}
        return result;
    }

    public long inputLong(String prompt)
    {  long result=0;
        try{result=Long.valueOf(input(prompt).trim()).longValue();}
        catch (Exception e){result = 0;}
        return result;
    }

    public double inputDouble(String prompt)
    {  double result=0;
        try{result=Double.valueOf(
                input(prompt).trim()).doubleValue();}
        catch (Exception e){result = 0;}
        return result;
    }

    public boolean inputBoolean(String prompt)
    {  boolean result=false;
        try{result=Boolean.valueOf(
                input(prompt).trim()).booleanValue();}
        catch (Exception e){result = false;}
        return result;
    }
    //============================================================
    //=========== end IBIO =======================================
    //============================================================

    java.awt.Font bigFont = new java.awt.Font("Verdana",java.awt.Font.BOLD,16);
    int useBigFonts = bigFonts();

    public String input(String prompt)
    { return javax.swing.JOptionPane.showInputDialog(null,prompt); }

    public void output(String message)
    { javax.swing.JOptionPane.showMessageDialog(null,message); }

    public int bigFonts()
    {
        javax.swing.UIManager.put(
            "OptionPane.messageFont",
            new javax.swing.plaf.FontUIResource(bigFont)
        );

        javax.swing.UIManager.put(
            "OptionPane.buttonFont",
            new javax.swing.plaf.FontUIResource(bigFont)
        );

        return 0;
    }
}