/********************************************\
This program inputs names and email addresses,
and stores these records in a Vector.
When the user types "quit", the program saves
the entire Vector (list) in a data file.
This is done by "serialization".
The next time the program runs, it automatically
loads all the data back from the file into
the Vector, and prints all the data in
the text console.  Then the user can add some
more records, type "quit" and save all the data
back into the data file.

Notice that each record is stored in an Email object.  
The Email class is in the same file with the EmailList class,
but could be stored in a separate file names "Email.java".

The first time you run the program, there will
be no data file on the disk, so you will see
an error message (File not found). 

Since the file name is written without a path,
Java will store the file in the same 
folder where the Java class is located.
\********************************************/

import java.util.*;
import java.io.*;

class Email implements Serializable
{
    String name = "";
    String address = "";
}

public class EmailList
{
    Vector<Email> emails = new Vector<Email>(); 

    public EmailList()
    {   
        loadData();
        showAllNames();
        inputData();
        saveData();
        showAllNames();
    }

    public void showAllNames()
    {  
        Iterator it = emails.iterator();

        while(it.hasNext())
        {   
            Email info = (Email)it.next();   
            System.out.println(info.name + "  " + info.address);
        }
        System.out.println("-------------------");
    }

    public void inputData()
    {
        String name = "go";
        while(!name.equals("quit"))
        {
            name = input("New Name (or 'quit')");
            if(!name.equals("quit"))
            {   // add a new record to the emails vector
                Email info = new Email();
                info.name = name;
                info.address = input("email address");
                emails.add(info);
            }
        }
    }  
    
    public void saveData()
    {
        try
        {
            FileOutputStream file = new FileOutputStream("emails.vec");
            ObjectOutputStream info = new ObjectOutputStream( file );
            info.writeObject( emails );
            info.close();
            file.close();
        }
        catch(Exception ex)
        {  output(ex.toString()); }
    }   
    
    public void loadData() 
    {
        try
        {
            FileInputStream file = new FileInputStream("emails.vec");
            ObjectInputStream info = new ObjectInputStream( file );
            emails = (Vector) info.readObject();
            info.close();
            file.close();
        }
        catch(Exception ex)
        { output(ex.toString()); }
    }    
    
    public static void main(String[] args)
    {  new EmailList(); }   
       
    public String input(String prompt)
    { return javax.swing.JOptionPane.showInputDialog(null,prompt); }

    public void output(String message)
    { javax.swing.JOptionPane.showMessageDialog(null,message); }
}

