/********************************************\
This program inputs names and email addresses,
and stores these records in a Vector.
When the user types "quit", the program saves
the entire Vector (list) in a data file.
This is done by "serialization".
The next time the program runs, it automatically
loads all the data back from the file into
the Vector, and prints all the data in
the text console.  Then the user can add some
more records, type "quit" and save all the data
back into the data file.

Notice that each record is stored in an 
Email object.  The Email class must be in 
a separate tab in Processing. To make the extra
tab, you must type "Email.java" as the name.

The first time you run the program, there will
be no data file on the disk, so you will see
an error message (File not found). 

Since the file name is written without a path,
Processing will store the file in the same 
folder where the Processing application is located.
\********************************************/


import java.util.*;
import java.io.*;

    Vector<Email> emails = new Vector<Email>(); 

    void setup()
    {   
        loadData();
        showAllNames();
        inputData();
        saveData();
    }

    public String input(String prompt)
    { return javax.swing.JOptionPane.showInputDialog(null,prompt); }

    public void output(String message)
    { javax.swing.JOptionPane.showMessageDialog(null,message); }

    public void showAllNames()
    {  
        Iterator it = emails.iterator();

        while(it.hasNext())
        {   
            Email info = (Email)it.next();   
            System.out.println(info.name + "  " + info.address);
        }
    }

    public void inputData()
    {
        String name = "go";
        while(!name.equals("quit"))
        {
            name = input("New Name (or 'quit')");
            if(!name.equals("quit"))
            {   // add a new record to the emails vector
                Email info = new Email();
                info.name = name;
                info.address = input("email address");
                emails.add(info);
            }
        }
    }  
    
    public void saveData()
    {
        try
        {
            FileOutputStream file = new FileOutputStream("emails.vec");
            ObjectOutputStream info = new ObjectOutputStream( file );
            info.writeObject( emails );
            info.close();
            file.close();
        }
        catch(Exception ex)
        {  output(ex.toString()); }
    }   
    
    public void loadData() 
    {
        try
        {
            FileInputStream file = new FileInputStream("emails.vec");
            ObjectInputStream info = new ObjectInputStream( file );
            emails = (Vector) info.readObject();
            info.close();
            file.close();
        }
        catch(Exception ex)
        { output(ex.toString()); }
    }    


