/*******************************************\
Processing contains special commands for 
saving an array of Strings into a text-file
on disk, and another for copying from the
text-file back into an array of Strings:
  saveStrings(filename);
  loadStrings(filename, array);
  
This program shows how to use these two commands.
It loads all the Strings from "dates.txt",
then adds a new String containing the current
date and time, and then saves the longer
list back into the same file. 

The first time the program runs, the data file
will not exist.  So the program contains code
to handle the corresponding errors.

If a simple file-name is used (without a path)
the program automatically saves the text file
in the same folder where the program is stored.
Use [Sketch/ShowSketchFolder] to see this
folder directly.
\*******************************************/

public class DatesList
{
  String[] dates;
  
  public DatesList()
  {
    loadDates();
    showAll(); 
    addDateTime();
    System.out.println("=========");
    showAll();
    saveDates();
  }
  
  void loadDates()
  {
    try
    {
      dates = loadStrings("dates.txt"); 
    }
    catch(Exception ex)
    {  System.out.println(ex.toString());}
  }
  
  void showAll()
  {
    if(dates != null)
    {
      for(int c = 0; c < dates.length; c = c+1)
      {
        System.out.println(c + " : " + dates[c]);
      }
    } 
  }
  
  void addDateTime()
  {
    String[] longer;
    if(dates == null)
    { longer = new String[1]; }
    else
    {  longer = new String[ dates.length + 1]; 
      for(int c = 0; c < dates.length; c = c+1)
      {
        longer[c] = dates[c];
      } 
    }
    
    longer[longer.length - 1] = (new java.util.Date()).toString();
    
    dates = longer;
  }
  
  void saveDates()
  {
    saveStrings("dates.txt",dates); 
  }
  
  String[] loadStrings(String fileName)
  {
    String[] array;
    try
    {
      java.io.BufferedReader countFile = new java.io.BufferedReader(
                                            new java.io.FileReader(fileName));
      int size = 0;
      while(countFile.ready())
      { countFile.readLine();
        size++;
      }
      countFile.close();
      array = new String[size];
      
      countFile = new java.io.BufferedReader(
                      new java.io.FileReader(fileName));
      int s = 0;
      while(countFile.ready())
      { array[s] = countFile.readLine();
        s++;
      }
      
      countFile.close();
      return array;
    }
    catch(Exception e)
    { System.out.println(e.toString());
      return null;
    }    
  }
  
  boolean saveStrings(String fileName,String[] data)
  {
    try
    { java.io.PrintWriter outfo = new java.io.PrintWriter(
                                     new java.io.FileWriter(fileName));
      int c = 0;
      while (c < data.length)
      { outfo.println(data[c]);
        c++;
      }
      outfo.close();
      return true;
    }
    catch(java.io.IOException e)
    { System.out.println(e.toString());
      return false;
    }
  }
  
  public static void main(String[] args)
  {  new DatesList(); }
}